/*!
 * @file network_stream.h
 *  implementation of a network pipe
 */

#ifndef _NETWORK_STREAM
#define _NETWORK_STREAM

#include <vector>
#include <list>
#include <map>

#include "data_stream.h"
#include "server_socket.h"
#include "handshake.h"
#include "connection_monitor.h"
#include "udp_server_socket.h"
#include "peer_info.h"

class Synchronizeable;
class NetworkSocket;
class ServerSocket;
class NetworkGameManager;


typedef std::list<Synchronizeable*>  SynchronizeableList;
typedef std::map<int,PeerInfo>       PeerList;


class NetworkStream : public DataStream
{

  public:
    NetworkStream();
    NetworkStream( std::string host, int port);
    NetworkStream( int port );

    virtual ~NetworkStream();
    void init();

    void createNetworkGameManager();
    void startHandshake();

    /* synchronizeable interface */
    void connectSynchronizeable(Synchronizeable& sync);
    void disconnectSynchronizeable(Synchronizeable& sync);

    /* functions for the localhost settings */
    inline bool isMasterServer() const { return (this->type == NET_MASTER_SERVER)? true:false; }
    inline bool isProxyServer() const { return (this->type == NET_PROXY_SERVER)? true:false; }
    inline bool isClient() const { return (this->type == NET_CLIENT)? true:false; }
//     inline bool isActive() const { return this->bActive; }
    inline int getMaxConnections(){ return MAX_CONNECTIONS; }

    /* functions for the peerInfo information retreival */
    inline bool isUserIdActive( int userID ) { return (peers.find(userID) != peers.end()); }
    inline bool isUserMasterServer( int userID ){ if ( !isUserIdActive(userID) ) return false; return peers[userID].isMasterServer; }
    inline bool isUserProxyServer( int userID ){ if ( !isUserIdActive(userID) ) return false; return peers[userID].isProxyServer; }
    inline bool isUserClient( int userID ){ if ( !isUserIdActive(userID) ) return false; return peers[userID].isClient; }


    virtual void processData();

    void debug();


  private:
    inline SynchronizeableList::const_iterator getSyncBegin(){ return synchronizeables.begin(); }
    inline SynchronizeableList::const_iterator getSyncEnd(){ return synchronizeables.end(); }
    inline PeerInfo & getPeerInfo( int userId ) { return peers[userId]; }

    int getSyncCount();

    void updateConnectionList();
    void handleHandshakes();
    void handleUpstream( int tick );
    void handleDownstream(int tick );
    void handleNewClient( int userId );
    void cleanUpOldSyncList();

    void writeToNewDict( byte * data, int length, bool upstream );


  private:
    PeerList                   peers;                       //!< list of the network node informations

    int                        type;                        //!< the type of this host (NET_CLIENT, NET_MASTER_SERVER,...)
    int                        myHostId;                    //!< the host id of the localhost

    std::list<int>             freeSocketSlots;             //!< list of free sockets (to ensure not to resycle sockets)
    int                        currentState;                //!< current state id

    NetworkGameManager*        networkGameManager;          //!< reference to the network game manager
    ServerSocket*              serverSocket;                //!< the listening socket of the server

    std::map<int,int>          oldSynchronizeables;         //!< used to save recently deleted sync ids to not recreate them
    SynchronizeableList        synchronizeables;            //!< list of the synchronizeables

    byte                       buf[UDP_PACKET_SIZE];        //!< used by handleUp/Downstream
    byte                       compBuf[UDP_PACKET_SIZE];    //!< used by handleUp/Downstream
    int                        remainingBytesToWriteToDict; //!< if > 0 NetworkStream will write packets to DATA/dicts/newdict

    int                        dictServer;                  //!< the zip dict for the server
    int                        dictClient;                  //!< the zip dict for the client
};
#endif /* _NETWORK_STREAM */
